@@ -9,7 +9,7 @@
 
 G_BEGIN_DECLS
 
-typedef enum
+typedef enum /*< flags >*/
 {
     HIPPO_PACK_EXPAND = 1,  /**< This is equivalent to both EXPAND and FILL for GtkBox,
                              * the way you'd get FILL=false is to set the alignment
@@ -111,7 +111,6 @@ struct _HippoCanvasBox {
     
     guint8 spacing;
 
-    guint floating : 1;
     guint needs_width_request : 1;
     guint needs_height_request : 1;
     guint needs_allocate : 1;
@@ -141,16 +140,16 @@ struct _HippoCanvasBoxClass {
 
     void     (* paint_background)             (HippoCanvasBox   *box,
                                                cairo_t          *cr,
-                                               HippoRectangle   *damaged_box);
+                                               GdkRegion        *damaged_region);
     void     (* paint_children)               (HippoCanvasBox   *box,
                                                cairo_t          *cr,
-                                               HippoRectangle   *damaged_box);
+                                               GdkRegion        *damaged_region);
     void     (* paint_below_children)         (HippoCanvasBox   *box,
                                                cairo_t          *cr,
-                                               HippoRectangle   *damaged_box);
+                                               GdkRegion        *damaged_region);
     void     (* paint_above_children)         (HippoCanvasBox   *box,
                                                cairo_t          *cr,
-                                               HippoRectangle   *damaged_box);
+                                               GdkRegion        *damaged_region);
     
     void     (* get_content_width_request)    (HippoCanvasBox   *box,
                                                int              *min_width_p,
