@@ -27,7 +27,7 @@ static void     hippo_canvas_image_set_context        (HippoCanvasItem    *item,
 /* Canvas box methods */
 static void hippo_canvas_image_paint_below_children       (HippoCanvasBox *box,
                                                            cairo_t        *cr,
-                                                           HippoRectangle  *damaged_box);
+                                                           GdkRegion      *damaged_region);
 static void hippo_canvas_image_get_content_width_request  (HippoCanvasBox *box,
                                                            int            *min_width_p,
                                                            int            *natural_width_p);
@@ -351,7 +351,7 @@ hippo_canvas_image_set_context(HippoCanvasItem    *item,
 static void
 hippo_canvas_image_paint_below_children(HippoCanvasBox  *box,
                                         cairo_t         *cr,
-                                        HippoRectangle  *damaged_box)
+                                        GdkRegion       *damaged_region)
 {
     HippoCanvasImage *image = HIPPO_CANVAS_IMAGE(box);
     int x, y, w, h;
