@@ -1,5 +1,6 @@
 /* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
 #include <string.h>
+#include <gdk/gdk.h>
 
 #include "hippo-canvas-internal.h"
 #include "hippo-canvas-item.h"
@@ -75,7 +76,7 @@ hippo_canvas_item_class_init(void *g_iface)
                       G_STRUCT_OFFSET(HippoCanvasItemIface, paint),
                       NULL, NULL,
                       hippo_canvas_marshal_VOID__POINTER_BOXED,
-                      G_TYPE_NONE, 2, G_TYPE_POINTER, HIPPO_TYPE_RECTANGLE);
+                      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_POINTER);
     /**
      * HippoCanvasItem::request-changed
      *
@@ -234,14 +235,6 @@ hippo_canvas_item_class_init(void *g_iface)
         
 }
 
-void
-hippo_canvas_item_sink(HippoCanvasItem    *canvas_item)
-{
-    g_return_if_fail(HIPPO_IS_CANVAS_ITEM(canvas_item));
-
-    HIPPO_CANVAS_ITEM_GET_IFACE(canvas_item)->sink(canvas_item);
-}
-
 /**
  * hippo_canvas_item_destroy:
  * @HippoCanvasItem: the canvas item
@@ -273,6 +266,10 @@ hippo_canvas_item_get_context(HippoCanvasItem *canvas_item)
     return HIPPO_CANVAS_ITEM_GET_IFACE(canvas_item)->get_context(canvas_item);
 }
 
+/**
+ * hippo_canvas_item_set_context:
+ * @context: (allow-none):
+ */
 void
 hippo_canvas_item_set_context(HippoCanvasItem    *canvas_item,
                               HippoCanvasContext *context)
@@ -662,38 +659,34 @@ hippo_canvas_item_process_event(HippoCanvasItem *canvas_item,
     return handled;
 }
 
-// the cairo_t and damaged_box are in the coordinate system in which canvas_item
+// the cairo_t and damaged_region are in the coordinate system in which canvas_item
 // is at (allocation_x, allocation_y), so they are both translated before
 // passing them to the child
 void
 hippo_canvas_item_process_paint(HippoCanvasItem *canvas_item,
                                 cairo_t         *cr,
-                                HippoRectangle  *damaged_box,
+                                GdkRegion       *damaged_region,
                                 int              allocation_x,
                                 int              allocation_y)
 {
-    HippoRectangle item_box;
-    HippoRectangle translated_box;
-    
+    GdkRectangle item_box;
+
     /* HippoCanvasItem::paint() is guaranteed to not be called if an item has any 0 allocation,
      * that invariant should be maintained here.
      */
-    
+
     item_box.x = allocation_x;
     item_box.y = allocation_y;
     hippo_canvas_item_get_allocation(canvas_item, &item_box.width, &item_box.height);
-    
-    if (hippo_rectangle_intersect(damaged_box, &item_box, &translated_box)) {
-        translated_box.x -= allocation_x;
-        translated_box.y -= allocation_y;
 
-        g_assert(translated_box.x >= 0);
-        g_assert(translated_box.y >= 0);
-        g_assert(translated_box.width > 0);
-        g_assert(translated_box.height > 0);
+    if (gdk_region_rect_in(damaged_region, &item_box) != GDK_OVERLAP_RECTANGLE_OUT) {
+        GdkRegion *translated_region;
+
+        translated_region = gdk_region_rectangle(&item_box);
+        gdk_region_intersect(translated_region, damaged_region);
+        gdk_region_offset(translated_region, -allocation_x, -allocation_y);
 
         cairo_save(cr);
-        
         cairo_translate(cr, allocation_x, allocation_y);
 
 #if 0
@@ -701,9 +694,11 @@ hippo_canvas_item_process_paint(HippoCanvasItem *canvas_item,
         cairo_rectangle(cr, 0, 0, item_box.width, item_box.height);
         cairo_fill(cr);
 #endif
-        
-        g_signal_emit(canvas_item, signals[PAINT], 0, cr, &translated_box);
-        
+
+        g_signal_emit(canvas_item, signals[PAINT], 0, cr, translated_region);
+
+        gdk_region_destroy(translated_region);
+
         cairo_restore(cr);
     }
 }
