@@ -37,6 +37,8 @@ struct _HippoCanvasStyle {
     double border_width[4];
     guint padding[4];
 
+    HippoCanvasThemeImage *background_theme_image;
+
     GType element_type;
     char *element_id;
     char *element_class;
@@ -48,6 +50,7 @@ struct _HippoCanvasStyle {
     guint borders_computed : 1;
     guint background_computed : 1;
     guint foreground_computed : 1;
+    guint background_theme_image_computed : 1;
     guint link_type : 2;
 };
 
@@ -100,6 +103,11 @@ hippo_canvas_style_finalize(GObject *object)
         style->font_desc = NULL;
     }
 
+    if (style->background_theme_image) {
+        g_object_unref(style->background_theme_image);
+        style->background_theme_image = NULL;
+    }
+
     G_OBJECT_CLASS(hippo_canvas_style_parent_class)->finalize(object);
 }
 
@@ -212,6 +220,88 @@ typedef enum {
     VALUE_INHERIT
 } GetFromTermResult;
 
+static int
+color_component_from_double(double component)
+{
+    /* http://people.redhat.com/otaylor/pixel-converting.html */
+    if (component >= 1.0)
+        return 255;
+    else
+        return (int)(component * 256);
+}
+
+static GetFromTermResult
+get_color_from_rgba_term(CRTerm           *term,
+                         guint32          *color)
+{
+    CRTerm *arg = term->ext_content.func_param;
+    CRNum *num;
+    double r = 0, g = 0, b = 0, a = 0;
+    int i;
+
+    for (i = 0; i < 4; i++) {
+        double value;
+            
+        if (arg == NULL)
+            return VALUE_NOT_FOUND;
+            
+        if ((i == 0 && arg->the_operator != NO_OP) ||
+            (i > 0 && arg->the_operator != COMMA))
+            return VALUE_NOT_FOUND;
+
+        if (arg->type != TERM_NUMBER)
+            return VALUE_NOT_FOUND;
+        
+        num = arg->content.num;
+
+        /* For simplicity, we convert a,r,g,b to [0,1.0] floats and then
+         * convert them back below. Then when we set them on a cairo content
+         * we convert them back to floats, and then cairo converts them
+         * back to integers to pass them to X, and so forth...
+         */
+        if (i < 3) {
+            if (num->type == NUM_PERCENTAGE) {
+                value = num->val / 100;
+            } else if (num->type == NUM_GENERIC) {
+                value = num->val / 255;
+            } else {
+                return VALUE_NOT_FOUND;
+            }
+        } else {
+            if (num->type != NUM_GENERIC)
+                return VALUE_NOT_FOUND;
+
+            value = num->val;
+        }
+
+        value = CLAMP(value, 0, 1);
+
+        switch (i) {
+        case 0:
+            r = value;
+            break;
+        case 1:
+            g = value;
+            break;
+        case 2:
+            b = value;
+            break;
+        case 3:
+            a = value;
+            break;
+        }
+        
+        arg = arg->next;
+    }
+
+    *color = ((color_component_from_double(r) << 24) |
+              (color_component_from_double(g) << 16) |
+              (color_component_from_double(b) << 8) |
+              ((color_component_from_double(a))));
+               
+    return VALUE_FOUND;
+}
+                         
 static GetFromTermResult
 get_color_from_term(HippoCanvasStyle *style,
                     CRTerm           *term,
@@ -220,6 +310,31 @@ get_color_from_term(HippoCanvasStyle *style,
     CRRgb rgb;
     enum CRStatus status;
 
+    /* Since libcroco doesn't know about rgba colors, it can't handle
+     * the transparent keyword
+     */
+    if (term->type == TERM_IDENT &&
+        term->content.str &&
+        term->content.str->stryng &&
+        term->content.str->stryng->str &&
+        strcmp(term->content.str->stryng->str, "transparent") == 0)
+    {
+        *color = 0;
+        return VALUE_FOUND;
+    }
+    /* rgba() colors - a CSS3 addition, are not supported by libcroco,
+     * but they are parsed as a "function", so we can emulate the
+     * functionality.
+     */
+    else if (term->type == TERM_FUNCTION &&
+             term->content.str &&
+             term->content.str->stryng &&
+             term->content.str->stryng->str &&
+             strcmp(term->content.str->stryng->str, "rgba") == 0)
+    {
+        return get_color_from_rgba_term(term, color);
+    }
+
     status = cr_rgb_set_from_term(&rgb, term);
     if (status != CR_OK)
         return VALUE_NOT_FOUND;
@@ -415,17 +530,20 @@ get_length_from_term(HippoCanvasStyle *style,
     case FONT_RELATIVE:
         {
             PangoFontDescription *desc;
+            double font_size;
             
             if (use_parent_font)
                 desc = get_parent_font(style);
             else
                 desc = hippo_canvas_style_get_font(style);
-            
+
+            font_size = (double)pango_font_description_get_size(desc) / PANGO_SCALE;
+
             if (pango_font_description_get_size_is_absolute(desc)) {
-                *length = num->val * multiplier * pango_font_description_get_size(desc);
+                *length = num->val * multiplier * font_size;
             } else {
                 double resolution = hippo_canvas_context_get_resolution(style->context);
-                *length = num->val * multiplier * (resolution / 72.) * pango_font_description_get_size(desc);
+                *length = num->val * multiplier * (resolution / 72.) * font_size;
             }
         }
         break;
@@ -728,7 +846,9 @@ hippo_canvas_style_get_background_color (HippoCanvasStyle *style)
             CRDeclaration *decl = style->properties[i];
             const char *property_name = decl->property->stryng->str;
             
-            if (!g_str_has_prefix(property_name, "background"))
+            if (g_str_has_prefix(property_name, "background"))
+                property_name += 10;
+            else
                 continue;
 
             if (strcmp(property_name, "") == 0) {
@@ -1270,6 +1390,112 @@ hippo_canvas_style_get_font (HippoCanvasStyle *style)
     return style->font_desc;
 }
 
+HippoCanvasThemeImage *
+hippo_canvas_style_get_background_theme_image (HippoCanvasStyle *style)
+{
+    int i;
+
+    if (style->background_theme_image_computed)
+        return style->background_theme_image;
+
+    style->background_theme_image = NULL;
+    style->background_theme_image_computed = TRUE;
+
+    ensure_properties(style);
+
+    for (i = style->n_properties - 1; i >= 0; i--) {
+        CRDeclaration *decl = style->properties[i];
+        
+        if (strcmp(decl->property->stryng->str, "-hippo-background-image") == 0) {
+            CRTerm *term = decl->value;
+            int lengths[4];
+            int n_lengths = 0;
+            int i;
+
+            const char *url;
+            int border_top;
+            int border_right;
+            int border_bottom;
+            int border_left;
+
+            char *filename;
+            GError *error = NULL;
+        
+            /* First term must be the URL to the image */
+            if (term->type != TERM_URI)
+                goto next_property;
+
+            url = term->content.str->stryng->str;
+            
+            term = term->next;
+
+            /* Followed by 0 to 4 lengths */
+            for (i = 0; i < 4; i++) {
+                double value;
+
+                if (term == NULL)
+                    break;
+
+                if (get_length_from_term(style, term, FALSE, &value) != VALUE_FOUND)
+                    goto next_property;
+
+                lengths[n_lengths] = (int)(0.5 + value);
+                n_lengths++;
+
+                term = term->next;
+            }
+
+            switch (n_lengths) {
+            case 0:
+                border_top = border_right = border_bottom = border_left = 0;
+                break; 
+            case 1:
+                border_top = border_right = border_bottom = border_left = lengths[0];
+                break; 
+            case 2:
+                border_top = border_bottom = lengths[0];
+                border_left = border_right = lengths[1];
+                break; 
+            case 3:
+                border_top = lengths[0];
+                border_left = border_right = lengths[1];
+                border_bottom = lengths[2];
+                break; 
+            case 4:
+            default:
+                border_top = lengths[0];
+                border_right = lengths[1];
+                border_bottom = lengths[2];
+                border_left = lengths[3];
+                break; 
+            }
+
+            filename = _hippo_canvas_theme_resolve_url(style->theme, decl->parent_statement->parent_sheet, url);
+            if (filename == NULL)
+                goto next_property;
+            
+            style->background_theme_image = hippo_canvas_theme_image_new(filename,
+                                                                         border_top, border_right, border_bottom, border_left,
+                                                                         &error);
+
+            g_free(filename);
+
+            if (style->background_theme_image == NULL) {
+                g_warning("Failed to load theme image: %s", error->message);
+                g_error_free(error);
+                goto next_property;
+            } else {
+                return style->background_theme_image;
+            }
+        }
+
+    next_property:
+        ;
+    }
+
+    return NULL;
+}
+
 gboolean
 hippo_canvas_style_paint (HippoCanvasStyle       *style,
                           cairo_t                *cr,
