@@ -4,6 +4,7 @@
 #include "hippo-canvas-style.h"
 #include "hippo-canvas-text.h"
 #include "hippo-canvas-box.h"
+#include <gdk/gdk.h>
 #include <pango/pangocairo.h>
 #include <stdlib.h>
 #include <string.h>
@@ -36,7 +37,7 @@ static char*    hippo_canvas_text_get_tooltip        (HippoCanvasItem    *item,
 /* Box methods */
 static void hippo_canvas_text_paint_below_children       (HippoCanvasBox *box,
                                                           cairo_t        *cr,
-                                                          HippoRectangle *damaged_box);
+                                                          GdkRegion      *damaged_region);
 static void hippo_canvas_text_get_content_width_request  (HippoCanvasBox *box,
                                                           int            *min_width_p,
                                                           int            *natural_width_p);
@@ -552,7 +553,7 @@ layout_is_ellipsized(PangoLayout *layout)
 static void
 hippo_canvas_text_paint_below_children(HippoCanvasBox  *box,
                                        cairo_t         *cr,
-                                       HippoRectangle  *damaged_box)
+                                       GdkRegion       *damaged_region)
 {
     HippoCanvasText *text = HIPPO_CANVAS_TEXT(box);
     guint32 color_rgba;
