@@ -17,6 +17,11 @@ void _hippo_canvas_theme_get_matched_properties (HippoCanvasTheme *theme,
                                                  CRDeclaration  ***properties,
                                                  int              *n_properties);
 
+/* Resolve an URL from the stylesheet to a filename */
+char *_hippo_canvas_theme_resolve_url (HippoCanvasTheme *theme,
+                                       CRStyleSheet     *base_stylesheet,
+                                       const char       *url);
+
 G_END_DECLS
 
 #endif /* __HIPPO_CANVAS_THEME_INTERNAL_H__ */
