@@ -468,15 +468,10 @@ hippo_canvas_helper_expose_event(HippoCanvasHelper *helper,
 
     if (helper->root != NULL) {
         int window_x, window_y;
-        HippoRectangle damage_box;
     
         get_root_item_window_coords(helper, &window_x, &window_y);
 
-        damage_box.x = event->area.x;
-        damage_box.y = event->area.y;
-        damage_box.width = event->area.width;
-        damage_box.height = event->area.height;
-        hippo_canvas_item_process_paint(helper->root, cr, &damage_box,
+        hippo_canvas_item_process_paint(helper->root, cr, event->region,
                                         window_x, window_y);
     }
     
@@ -1329,8 +1324,7 @@ hippo_canvas_helper_set_root(HippoCanvasHelper *helper,
     }
 
     if (root != NULL) {
-        g_object_ref(root);
-        hippo_canvas_item_sink(root);
+        g_object_ref_sink(root);
         helper->root = root;
         g_signal_connect(root, "destroy",
                          G_CALLBACK(canvas_root_destroy),
