@@ -26,6 +26,10 @@ GtkWidget* hippo_canvas_window_new    (void);
 void hippo_canvas_window_set_root(HippoCanvasWindow *canvas_window,
                                   HippoCanvasItem   *item);
 
+void hippo_canvas_window_set_theme(HippoCanvasWindow *canvas_window,
+                                   HippoCanvasTheme  *theme);
+
+HippoCanvasContext *hippo_canvas_window_get_context(HippoCanvasWindow *canvas_window);
 
 G_END_DECLS
 
