@@ -19,18 +19,6 @@ void pyhippo_add_constants(PyObject *module, const gchar *strip_prefix);
 
 extern PyMethodDef pyhippo_functions[];
 
-/* this is analogous to what pygtk does for GtkObject, GtkWindow, etc.
- * but it just has to be a bug in pygtk that it works this way.
- */
-static void
-sink_hippocanvasbox(GObject *object)
-{
-    if (HIPPO_CANVAS_BOX(object)->floating) {
-        g_object_ref(object);
-        hippo_canvas_item_sink(HIPPO_CANVAS_ITEM(object));
-    }
-}
-
 static PyObject *
 _cairo_surface_from_gvalue(const GValue *value)
 {
@@ -59,8 +47,6 @@ inithippo(void)
     m = Py_InitModule("hippo", pyhippo_functions);
     d = PyModule_GetDict(m);
 
-    pygobject_register_sinkfunc(HIPPO_TYPE_CANVAS_BOX, sink_hippocanvasbox);
-
     pyhippo_register_classes(d);
     pyhippo_add_constants(m, "HIPPO_");
     PyModule_AddObject(m, "TYPE_CAIRO_SURFACE", pyg_type_wrapper_new(HIPPO_TYPE_CAIRO_SURFACE));
